#include "mlpproblem.h"

MlpProblem::MlpProblem(int n,Dataset *d)
    :Problem((d->dimension()+2)*n)
{
    data = d;
    int k = (d->dimension()+2)*n;
    left.resize(k);
    right.resize(k);
    for(int i=0;i<k;i++)
    {
        left[i]=-10;
        right[i]=10;
    }
    weight.resize(k);
}

double MlpProblem::funmin(Data &x)
{
    weight  =x ;
    double error = 0.0;
    for(int i=0;i<data->count();i++)
    {
        Data xx = data->getXPoint(i);
        double yy = data->getYPoint(i);
        double per = getOutput(xx);
        error+= (per-yy)*(per-yy);
    }
    return error;
}

Data    MlpProblem::gradient(Data &x)
{
    Data g;
    weight = x;
    g.resize(weight.size());
    for(int i=0;i<g.size();i++)
        g[i]=0.0;
    for(int i=0;i<data->count();i++)
    {
        Data xx = data->getXPoint(i);
        Data gtemp = getDerivative(xx);
        double per=getOutput(xx)-data->getYPoint(i);
        for(int j=0;j<g.size();j++)	g[j]+=gtemp[j]*per;
    }
    for(int j=0;j<x.size();j++) g[j]*=2.0;
    return g;
}

double  MlpProblem::sig(double x)
{
    return 1.0/(1.0+exp(-x));
}

double  MlpProblem::sigder(double x)
{
    double s = sig(x);
    return s*(1.0-s);
}

double  MlpProblem::getOutput(Data  &x)
{
    double arg=0.0;
    double per=0.0;
    int nodes = weight.size()/(data->dimension()+2);
     int d = data->dimension();
    for(int i=1;i<=nodes;i++)
    {
        arg=0.0;
        for(int j=1;j<=d;j++)
        {
            int pos=(d+2)*i-(d+1)+j-1;
            arg+=weight[pos]*x[j-1];
        }
        arg+=weight[(d+2)*i-1];
        per+=weight[(d+2)*i-(d+1)-1]*sig(arg);
    }
    return per;
}

Data    MlpProblem::getDerivative(Data &x)
{
    double arg;
        double f,f2;
        int nodes = weight.size()/(data->dimension()+2);
        int d = data->dimension();
        Data G;
        G.resize(weight.size());

        for(int i=1;i<=nodes;i++)
        {
                arg = 0.0;
                for(int j=1;j<=d;j++)
                {
                        arg+=weight[(d+2)*i-(d+1)+j-1]*x[j-1];
                }
                arg+=weight[(d+2)*i-1];
                f=sig(arg);
                f2=f*(1.0-f);
                G[(d+2)*i-1]=weight[(d+2)*i-(d+1)-1]*f2;
                G[(d+2)*i-(d+1)-1]=f;
                for(int k=1;k<=d;k++)
                {
                        G[(d+2)*i-(d+1)+k-1]=
                                x[k-1]*f2*weight[(d+2)*i-(d+1)-1];
                }
    }
        return G;
}

MlpProblem::~MlpProblem()
{
    //nothing here
}
